# Window.py -- Pamhyr
# Copyright (C) 2023  INRAE
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-

import logging

from tools import trace, timer

from View.ASubWindow import ASubMainWindow
from View.ListedSubWindow import ListedSubWindow

from PyQt5.QtGui import (
    QKeySequence,
)

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox, QVBoxLayout, QHeaderView, QTabWidget,
    QSlider,
)

from View.Plot.MplCanvas import MplCanvas

from View.Results.PlotXY import PlotXY
from View.Results.PlotAC import PlotAC
from View.Results.PlotKPC import PlotKPC

from View.Results.Table import TableModel
from View.Results.translate import *
from View.Stricklers.Window import StricklersWindow

_translate = QCoreApplication.translate

logger = logging.getLogger()

class ResultsWindow(ASubMainWindow, ListedSubWindow):
    def __init__(self, title="Results",
                 study=None, solver=None, results=None,
                 parent=None):
        self._study = study
        self._solver = solver
        self._results = results

        self._timestamps = sorted(list(self._results.get("timestamps")))

        self.setup_title(title)

        super(ResultsWindow, self).__init__(
            name=title, ui="Results", parent=parent
        )

        self.setup_sc()
        self.setup_table()
        self.setup_graph()
        self.setup_slider()
        self.setup_connections()

        self.ui.setWindowTitle(self._title)

    def setup_title(self, title):
        self._title = (
            title + " - "
            + self._study.name + " - "
            + self._solver.name + " - "
            + self._results.date
        )

    def setup_sc(self):
        self._undo_stack = QUndoStack()

        self.undo_sc = QShortcut(QKeySequence.Undo, self)
        self.redo_sc = QShortcut(QKeySequence.Redo, self)
        self.copy_sc = QShortcut(QKeySequence.Copy, self)
        self.paste_sc = QShortcut(QKeySequence.Paste, self)

    def setup_table(self):
        self._table = {}

        for t in ["reach", "profile"]:
            table = self.find(QTableView, f"tableView_{t}")
            self._table[t] = TableModel(
                results = self._results,
                study = self._study,
                mode = t,
                undo = self._undo_stack,
            )
            table.setModel(self._table[t])

            table.setSelectionBehavior(QAbstractItemView.SelectRows)
            table.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
            table.setAlternatingRowColors(True)

    def setup_slider(self):
        self._slider_profile = self.find(QSlider, f"verticalSlider_profile")
        default_reach = self._results.river.reach(0)
        self._slider_profile.setMaximum(len(default_reach.profiles) - 1)
        self._slider_profile.setValue(0)

        self._slider_time = self.find(QSlider, f"horizontalSlider_time")
        self._slider_time.setMaximum(len(self._timestamps) - 1)
        self._slider_time.setValue(len(self._timestamps) - 1)

    def setup_graph(self):
        self.canvas = MplCanvas(width=5, height=4, dpi=100)
        self.canvas.setObjectName("canvas")
        self.plot_layout = self.find(QVBoxLayout, "verticalLayout")
        self.plot_layout.addWidget(self.canvas)

        self.plot_xy = PlotXY(
            canvas = self.canvas,
            results = self._results,
            reach_id = 0,
            profile_id = 0,
            toolbar = None,
            display_current = False
        )
        self.plot_xy.draw()

        self.canvas_2 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_2.setObjectName("canvas_2")
        self.plot_layout_2 = self.find(QVBoxLayout, "verticalLayout_2")
        self.plot_layout_2.addWidget(self.canvas_2)

        self.plot_kpc = PlotKPC(
            canvas = self.canvas_2,
            results = self._results,
            reach_id = 0,
            profile_id = 0,
            toolbar = None
        )
        self.plot_kpc.draw()

        self.canvas_3 = MplCanvas(width=5, height=4, dpi=100)
        self.canvas_3.setObjectName("canvas_3")
        self.plot_layout_3 = self.find(QVBoxLayout, "verticalLayout_3")
        self.plot_layout_3.addWidget(self.canvas_3)

        self.plot_ac = PlotAC(
            canvas = self.canvas_3,
            results = self._results,
            reach_id = 0,
            profile_id = 0,
            toolbar = None
        )
        self.plot_ac.draw()

    def setup_connections(self):
        self.undo_sc.activated.connect(self.undo)
        self.redo_sc.activated.connect(self.redo)
        self.copy_sc.activated.connect(self.copy)
        self.paste_sc.activated.connect(self.paste)

        fun = {
            "reach": self._set_current_reach,
            "profile": self._set_current_profile,
        }

        for t in ["reach", "profile"]:
            table = self.find(QTableView, f"tableView_{t}")

            table.selectionModel()\
                 .selectionChanged\
                 .connect(fun[t])

            self._table[t].dataChanged.connect(fun[t])

        self._slider_profile.valueChanged.connect(self._set_current_profile_slider)
        self._slider_time.valueChanged.connect(self._set_current_timestamp)


    def update(self, reach_id = None, profile_id = None, timestamp = None):
        if reach_id is not None:
            self.plot_xy.set_reach(reach_id)
            self.plot_ac.set_reach(reach_id)
            self.plot_kpc.set_reach(reach_id)
        if profile_id is not None:
            self.plot_xy.set_profile(profile_id)
            self.plot_ac.set_profile(profile_id)
            self.plot_kpc.set_profile(profile_id)
        if timestamp is not None:
            self.plot_xy.set_timestamp(timestamp)
            self.plot_ac.set_timestamp(timestamp)
            self.plot_kpc.set_timestamp(timestamp)

        self.plot_xy.draw()
        self.plot_ac.draw()
        self.plot_kpc.draw()


    def _set_current_reach(self):
        table = self.find(QTableView, f"tableView_reach")
        indexes = table.selectedIndexes()

        self.update(reach_id = indexes[0].row())

    def _set_current_profile(self):
        table = self.find(QTableView, f"tableView_profile")
        indexes = table.selectedIndexes()

        ind = indexes[0].row()
        self.update(profile_id = ind)
        self._slider_profile.setValue(ind)

    def _set_current_profile_slider(self):
        pid = self._slider_profile.value()
        self.update(profile_id = pid)

        return

    def _set_current_timestamp(self):
        timestamp = self._timestamps[self._slider_time.value()]
        self.update(timestamp = timestamp)

    def copy(self):
        logger.info("TODO: copy")

    def paste(self):
        logger.info("TODO: paste")

    def undo(self):
        self._table.undo()

    def redo(self):
        self._table.redo()
